// repeats what you do
// ATtiny85 at 8 MHz 
//https://bitbucket.org/mrgrok/mrgroks-arduino-sketches/src/b2d52c93640c9f582ac0cd0e867c2a67107274f4/recordAndPlaybackAtRate/recordAndPlaybackAtRate.ino?fileviewer=file-view-default


const byte morseBtn = 4;
const byte modeBtn = 3;


const byte led = 0; //ATtiny85
const byte sampleLength = 1; // 1 ms
const byte maxSamples = 100;

boolean inRecordMode = false;
boolean wasInRecordMode = false;

// buffers to record state/duration combinations
boolean states[maxSamples];
int durations[maxSamples];
int currentSampleCycles;

short idxPlayback = 0;
short idxRecord = 0;

void setup() {
  pinMode(modeBtn, INPUT);
  pinMode(morseBtn, INPUT);
  pinMode(led, OUTPUT);
}

void resetForRecording() {
  memset(states, 0, sizeof(states));
  memset(durations, 0, sizeof(durations));
  idxRecord = 0; // reset record idx just to make playback start point obvious 
  
  idxPlayback=0; 
  currentSampleCycles=0;
}

void loop() {
  inRecordMode = digitalRead(modeBtn);
  if(inRecordMode == true) {
    if(!wasInRecordMode) {
      resetForRecording();      
    }
    recordLoop();
  } else {
    // continue playing loop
    playbackLoop();
  }
  
  wasInRecordMode = inRecordMode; // record prev state for next iteration so we know whether to reset the record arr index
}

void recordLoop() {
  boolean state = digitalRead(morseBtn);
  digitalWrite(led, state); // give feedback to person recording the loop
  
  if(states[idxRecord] == state) {
    // state not changed, add to duration of current state
    durations[idxRecord] += sampleLength;
  } else {
    // state changed, go to next idx and set default duration
    idxRecord++;
    if(idxRecord == maxSamples) { idxRecord = 0; } // reset idx if max array size reached
    states[idxRecord] = state;
    durations[idxRecord] = sampleLength;
  }
  
  delay(sampleLength); // slow the loop to a time constant so we can reproduce the timelyness of the recording
}

void playbackLoop()
{
  

  if(currentSampleCycles == 0 && durations[idxPlayback] != 0) {
    // state changed
    digitalWrite(led, states[idxPlayback]); // set led
  }
  
  if(idxPlayback == maxSamples) { 
    // EOF recorded loop - repeat
    idxPlayback=0; 
    currentSampleCycles=0;
  } else {
  //  if(durations[idxPlayback]*playbackMultiplier <= currentSampleCycles) // speed ctls deactivated
    if(durations[idxPlayback]*1 <= currentSampleCycles) 
    {
      // EOF current sample in recorder buffer 
      idxPlayback++; // move to next sample 
      currentSampleCycles = 0; // reset for next sample
    } else {
      // still in same sample, but in next cycle (==sampleLength approx)
      currentSampleCycles++;
    }
  }
  
  delay(sampleLength); // keep time same as we had for record loop (approximately) 
}
